from pathlib import Path, PosixPath

from django.apps import AppConfig
from django.core.checks import Warning, register
from django.template.exceptions import TemplateDoesNotExist
from django.template.loader import get_template
from django.utils.autoreload import autoreload_started, file_changed

import yaml

from slippers.templatetags.slippers import (
    register_block_components,
    register_inline_components,
)


def register_tags():
    """Register tags from components.yaml"""
    try:
        template = get_template("components.yaml")
        components = yaml.safe_load(template.template.source)
        register_inline_components(components.get("inline_components", {}))
        register_block_components(components.get("block_components", {}))
    except TemplateDoesNotExist:
        pass


def watch(sender, **kwargs):
    """Watch when component.yaml changes"""
    try:
        template = get_template("components.yaml")
        sender.extra_files.add(Path(template.origin.name))
    except TemplateDoesNotExist:
        pass


def changed(sender, file_path: PosixPath, **kwargs):
    """Refresh tag registry when component.yaml changes"""
    if file_path.name == "components.yaml":
        print("components.yaml changed. Updating component tags...")
        register_tags()


def checks(app_configs, **kwargs):
    """Warn if unable to find components.yaml"""
    try:
        get_template("components.yaml")
    except TemplateDoesNotExist:
        return [
            Warning(
                "Slippers was unable to find a components.yaml file.",
                hint="Make sure it's in a root template directory.",
                id="slippers.E001",
            )
        ]
    return []


class SlippersConfig(AppConfig):
    name = "slippers"

    def ready(self):
        register_tags()

        register(checks)

        autoreload_started.connect(watch)
        file_changed.connect(changed)



from typing import Dict

from django import template
from django.template import Context
from django.template.base import token_kwargs

register = template.Library()


def do_inline_component(**kwargs):
    return kwargs


def create_block_component_tag(template_path):
    def do_block_component(parser, token):
        tag_name, *remaining_bits = token.split_contents()
        extra_context = token_kwargs(remaining_bits, parser)
        nodelist = parser.parse((f"end{tag_name}",))
        parser.delete_first_token()
        return BlockComponentNode(nodelist, template_path, extra_context)

    return do_block_component


class BlockComponentNode(template.Node):
    def __init__(self, nodelist, template, extra_context):
        self.nodelist = nodelist
        self.template = template
        self.extra_context = extra_context

    def render(self, context):
        children = self.nodelist.render(context)
        t = context.template.engine.get_template(self.template)

        values = {key: val.resolve(context) for key, val in self.extra_context.items()}

        return t.render(
            Context({**values, "children": children}, autoescape=context.autoescape)
        )


def register_inline_components(components: Dict[str, str]) -> None:
    for tag_name, template_path in components.items():
        # Inline components use Django's built-in `inclusion_tag`
        register.inclusion_tag(name=tag_name, filename=template_path)(
            do_inline_component
        )


def register_block_components(components: Dict[str, str]) -> None:
    for tag_name, template_path in components.items():
        register.tag(tag_name, create_block_component_tag(template_path))



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['slippers', 'slippers.templatetags']

package_data = \
{'': ['*']}

install_requires = \
['Django>=2.2,<4.0', 'PyYAML>=5.4.1,<6.0.0']

setup_kwargs = {
    'name': 'slippers',
    'version': '0.1.4',
    'description': 'Slippers allows you to build reusable components in Django without writing a single line of Python.',
    'long_description': '# Slippers\n\n[![PyPI version](https://badge.fury.io/py/slippers.svg)](https://badge.fury.io/py/slippers)\n[![PyPI Supported Python Versions](https://img.shields.io/pypi/pyversions/slippers.svg)](https://pypi.python.org/pypi/slippers/)\n[![GitHub Actions (Code quality and tests)](https://github.com/mixxorz/slippers/workflows/Code%20quality%20and%20tests/badge.svg)](https://github.com/mixxorz/slippers)\n\nSlippers allows you to build reusable components in Django without writing a single line of Python.\n\n```django\n{% card variant="small" %}\n  <h1>Slippers is cool</h1>\n\n  {% button %}Super cool{% endbutton %}\n  {% button variant="secondary" %}Lit af{% endbutton %}\n{% endcard %}\n```\n\n## Why?\n\nI want to be able to make reusable components, but the syntax for `{% include %}` is too verbose. Plus it doesn\'t allow me to specify child elements.\n\n## Show me how it works\n\nFirst create your template. Wherever you would normally put it is fine.\n\n```django\n{# myapp/templates/myapp/card.html #}\n<div class="card">\n  <h1 class="card__header">{{ heading }}</h1>\n  <div class="card__body">\n    {# Child elements are rendered by `{{ children }}` #}\n    {{ children }}\n  </div>\n</div>\n```\n\nNext, create a `components.yaml` file. By default, Slippers looks for this file in the root template folder.\n\n```yaml\n# myapp/templates/components.yaml\n# Components that have child elements\nblock_components:\n  card: "myapp/card.html"\n \n# Components that don\'t have child elements\ninline_components: \n  avatar: "myapp/avatar.html"\n```\n\nYou can now use the components like so:\n\n```django\n{% load slippers %}\n\n{% card heading="Slippers is awesome" %}\n  <span>Hello {{ request.user.full_name }}!</span>\n{% endcard %}\n```\n\nAnd the output:\n\n```html\n<div class="card">\n  <h1 class="card__header">Slippers is awesome</h1>\n  <div class="card__body">\n    <span>Hello Ryland Grace!</span>\n  </div>\n</div>\n```\n\n## Installation\n\n```\npip install slippers\n```\n\nAdd it to your `INSTALLED_APPS`:\n\n```python\nINSTALLED_APPS = [\n    ...\n    \'slippers\',\n    ...\n]\n```\n\n## Documentation\n\n### The `components.yaml` file\n\nThis file should be placed at the root template directory. E.g. `myapp/templates/components.yaml`.\n\nThe structure of the file is as follows:\n\n```yaml\n# Components that have child elements are called "block" components\nblock_components:\n  # The key determines the name of the template tag. So `card` would generate\n  # `{% card %}{% endcard %}`\n  # The value is the path to the template file as it would be if used with {% include %}\n  card: "myapp/card.html"\n \n# Components that don\'t have child elements are called "inline" components\ninline_components: \n  avatar: "myapp/avatar.html"\n```\n\nThis file also doubles as an index of available components which is handy.\n\n### Context\n\nUnlike `{% include %}`, using the component template tag **will not** pass the\ncurrent context to the child component. This is a design decision. If you need\nsomething from the parent context, you have to explicitly pass it in via keyword\narguments, or use `{% include %}` instead.\n\n```django\n{% with not_passed_in="Lorem ipsum" %}\n  {% button is_passed_in="Dolor amet" %}Hello{% endbutton %}\n{% endwith %}\n```\n\n## License\n\nMIT\n',
    'author': 'Mitchel Cabuloy',
    'author_email': 'mixxorz@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/mixxorz/slippers',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.6.2,<4.0.0',
}


setup(**setup_kwargs)
